# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import tracker.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Tracker',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('modified_at', models.DateTimeField(auto_now=True, verbose_name='Modified at')),
                ('data', tracker.models.DictField(null=True, blank=True)),
                ('user', models.ForeignKey(related_name='tracked_user', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
        ),
    ]


from django.db import models
from django.core.serializers.json import DjangoJSONEncoder
from djutil.models import TimeStampedModel
from django.conf import settings

import json


# New custom data-type django, use this if you want to store a dict !
class DictField(models.TextField):
    """
    JSONField is a generic textfield that neatly serializes/unserializes
    JSON objects seamlessly.
    Django snippet #1478

    example:
        class Page(models.Model):
            data = JSONField(blank=True, null=True)


        page = Page.objects.get(pk=5)
        page.data = {'title': 'test', 'type': 3}
        page.save()
    """

    __metaclass__ = models.SubfieldBase

    def to_python(self, value):
        if value == "":
            return None

        try:
            if isinstance(value, basestring):
                return json.loads(value)
        except ValueError:
            # Handle appropriately
            pass
        return value

    def get_db_prep_save(self, value, *args, **kwargs):
        if value == "":
            return None
        if isinstance(value, dict):
            value = json.dumps(value, cls=DjangoJSONEncoder)
        return super(DictField, self).get_db_prep_save(value, *args, **kwargs)


class Tracker(TimeStampedModel):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="tracked_user", null=True, blank=True)
    data = DictField(null=True, blank=True)

    class Meta:
        app_label = 'core'




from tracker.models import Tracker


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def track_this(view):
    def tracker(request, *args, **kwargs):
        data = {
            'path': request.path,
            'method': request.method,
            'ip_address': get_client_ip(request),
            'user_agent': request.META['HTTP_USER_AGENT']
        }
        try:
            user = request.user if not request.user.is_anonymous() else None
            Tracker.objects.create(user=user, data=data)
        except Exception:
            print 'Error ! need a logger here :)'

        return view(request, *args, **kwargs)
    return tracker


# Create your views here.
